@@ -4,6 +4,7 @@
 import gzip
 import os
 import re
+import shutil
 import subprocess
 from pathlib import Path
 from typing import Dict, List, Optional
@@ -13,6 +14,10 @@ import requests
 from Bio import SeqIO
 
 
+class ShellCommandError(Exception):
+    """Error for shell utilities that aren't installed."""
+
+
 def fastq_to_list(filepath: Path) -> Optional[List[SeqIO.SeqRecord]]:
     """Read a .fastq or fastq.gz file into an in-memory record_list.
 
@@ -70,10 +75,16 @@ def build_bowtie2_index(fasta: Path) -> str:
 
     Returns:
         str: output from bowtie2-build shell command, usually discarded
+
+    Raises:
+        ShellCommandError: bowtie2-build is not installed.
     """
 
     command: List[str] = ["bowtie2-build", "index", os.fspath(fasta)]
 
+    if shutil.which(command[0]) is None:
+        raise ShellCommandError("bowtie2-build is not installed.")
+
     output: subprocess.CompletedProcess[str] = subprocess.run(
         command, capture_output=True, text=True, check=True
     )
@@ -92,6 +103,9 @@ def bowtie2_fastq_to_bam(index: Path, fastq: Path, cores: int) -> str:
 
     Returns:
         str: output from bowtie2 and samtools shell command, usually discarded
+
+    Raises:
+        ShellCommandError: bowtie2 is not installed.
     """
 
     command: List[str] = [
@@ -111,6 +125,9 @@ def bowtie2_fastq_to_bam(index: Path, fastq: Path, cores: int) -> str:
         str(fastq.parent) + "/" + str(fastq.stem) + ".bam",
     ]
 
+    if shutil.which(command[0]) is None:
+        raise ShellCommandError("bowtie2 is not installed.")
+
     output: subprocess.CompletedProcess[str] = subprocess.run(
         command, capture_output=True, text=True, check=True
     )
@@ -119,7 +136,7 @@ def bowtie2_fastq_to_bam(index: Path, fastq: Path, cores: int) -> str:
     return out
 
 
-def pysam_sort(bamfile: Path) -> bool:
+def sort_bam(bamfile: Path) -> bool:
     """Helper function to run pysam samtools sort.
 
     Args:
@@ -134,7 +151,7 @@ def pysam_sort(bamfile: Path) -> bool:
     return True
 
 
-def pysam_index(bamfile: Path) -> bool:
+def index_bam(bamfile: Path) -> bool:
     """Helper function to run external samtools index.
 
     Args:
@@ -149,7 +166,7 @@ def pysam_index(bamfile: Path) -> bool:
     return True
 
 
-def retrieve_promoter_seq(filename: str, chrom: str, start: int, end: int) -> None:
+def retrieve_seq(filename: str, chrom: str, start: int, end: int) -> None:
     """Retrieve the genomic sequence of interest from UCSC Genome Browser.
 
     Args:
@@ -182,6 +199,9 @@ def prepare_genome(index: Path, aligner: Optional[Path] = None) -> str:
 
     Returns:
         str: output from genome preparation shell command, usually discarded
+
+    Raises:
+        ShellCommandError: bismark_genome_preparation is not installed.
     """
     command: List[str]
     if aligner:
@@ -197,6 +217,9 @@ def prepare_genome(index: Path, aligner: Optional[Path] = None) -> str:
             str(index),
         ]
 
+    if shutil.which(command[0]) is None:
+        raise ShellCommandError("bismark_genome_preparation is not installed.")
+
     output: subprocess.CompletedProcess[str] = subprocess.run(
         command,
         capture_output=True,
@@ -221,6 +244,9 @@ def bismark(genome: Path, fastq: Path) -> str:
 
     Returns:
         str: output from bismark shell command, usually discarded
+
+    Raises:
+        ShellCommandError: bismark is not installed.
     """
 
     command: List[str] = [
@@ -230,6 +256,9 @@ def bismark(genome: Path, fastq: Path) -> str:
         str(fastq),
     ]
 
+    if shutil.which(command[0]) is None:
+        raise ShellCommandError("bismark is not installed.")
+
     output: subprocess.CompletedProcess[str] = subprocess.run(
         command,
         capture_output=True,
