@@ -3,10 +3,10 @@
    in reduced representation bisulfate DNA sequencing.
 """
 
+import copy
 import pickle  # nosec
 import re
 import signal
-import warnings
 from multiprocessing import Pool, cpu_count
 from multiprocessing.pool import AsyncResult
 from pathlib import Path
@@ -17,13 +17,13 @@ import numpy as np
 import pandas as pd
 import plotly.graph_objects as go
 import pysam
-from numpy.typing import ArrayLike
+from numpy.typing import ArrayLike, NDArray
 from scipy import stats
 from tqdm.auto import tqdm
 
-from . import quma
-from . import visualization as viz
-from .config import Config
+from pyllelic import quma
+from pyllelic import visualization as viz
+from pyllelic.config import Config
 
 # Initialized multiprocessing limits
 NUM_THREADS = cpu_count() - 1
@@ -317,7 +317,12 @@ class GenomicPositionData:
         self.quma_results: Dict[str, QumaResult] = self._quma_full_threaded()
         """Dict[str, QumaResult]: list of QumaResults."""
 
-        with tqdm(total=4, desc="Summary Statistics") as pbar:
+        with tqdm(total=6, desc="Summary Statistics") as pbar:
+            self._individual_position_data: Dict[
+                str, pd.DataFrame
+            ] = self._create_individual_position_df_dict()
+            pbar.update(1)
+
             self.means: pd.DataFrame = self._process_means()
             """pd.DataFrame: dataframe of mean methylation values."""
             pbar.update(1)
@@ -332,6 +337,10 @@ class GenomicPositionData:
 
             self.individual_data: pd.DataFrame = self._return_individual_data()
             """pd.DataFrame: dataframe of individual methylation values."""
+            pbar.update(1)
+
+            self.allelic_data: pd.DataFrame = self._generate_chisquared_test_df()
+            """pd.DataFrame: dataframe of Chi-squared p-values."""
 
             self.positions = self.means.columns.tolist()
             pbar.update(1)
@@ -458,6 +467,18 @@ class GenomicPositionData:
 
         return data
 
+    def _create_individual_position_df_dict(self) -> Dict[str, pd.DataFrame]:
+        """Create a dictionary of invidual position data for use in aggregate dfs.
+
+        Returns:
+            Dict[str, pd.DataFrame]: dictionary of invidual position data dfs
+        """
+        df_dict: Dict[str, pd.DataFrame] = {
+            cell: self._return_individual_positions(cell).apply(pd.to_numeric)
+            for cell in self.cell_types
+        }
+        return df_dict
+
     def _process_means(self) -> pd.DataFrame:
         """Process the mean values at each individual position for each cell line.
 
@@ -465,12 +486,11 @@ class GenomicPositionData:
             pd.DataFrame: dataframes of mean values for each position in each cell line.
         """
 
-        df_list = {
-            cell: self.return_individual_positions(cell).apply(pd.to_numeric).mean()
-            for cell in self.cell_types
-        }
+        df_dict: Dict[str, pd.DataFrame] = copy.deepcopy(self._individual_position_data)
+        for k, v in df_dict.items():
+            df_dict[k] = v.mean()
 
-        df = pd.DataFrame.from_dict(df_list).T
+        df: pd.DataFrame = pd.DataFrame.from_dict(df_dict).transpose()
         df.columns = df.columns.astype(str)
         return df
 
@@ -481,15 +501,11 @@ class GenomicPositionData:
             pd.DataFrame: dataframes of mode values for each individual position.
         """
 
-        df_list = {
-            cell: self.return_individual_positions(cell)
-            .apply(pd.to_numeric)
-            .mode()
-            .iloc[0]
-            for cell in self.cell_types
-        }
+        df_dict: Dict[str, pd.DataFrame] = copy.deepcopy(self._individual_position_data)
+        for k, v in df_dict.items():
+            df_dict[k] = v.mode().iloc[0]
 
-        df = pd.DataFrame.from_dict(df_list).T
+        df: pd.DataFrame = pd.DataFrame.from_dict(df_dict).transpose()
         df.columns = df.columns.astype(str)
         return df
 
@@ -500,26 +516,29 @@ class GenomicPositionData:
             pd.DataFrame: methylation values for each position in each cell line
         """
 
-        df_list: List[pd.DataFrame] = []
-        for cell in self.cell_types:
-            ind_pos: pd.DataFrame = self.return_individual_positions(cell).apply(
-                pd.to_numeric
-            )
-            ind_pos = ind_pos.stack().reset_index(level=0, drop=True)
-            ind_pos = (
-                ind_pos.groupby(ind_pos.index)
+        df_dict: Dict[str, pd.DataFrame] = copy.deepcopy(self._individual_position_data)
+        for k, v in df_dict.items():
+            df_dict[k] = (
+                (v.stack().reset_index(level=0, drop=True))
+                .groupby(level=0)
                 .apply(list)
                 .to_frame()
                 .transpose()
-                .rename(index={0: cell})
             )
 
-            df_list.append(ind_pos)
-
-        df = pd.concat(df_list)
+        df: pd.DataFrame = pd.concat(df_dict)
         df.columns = df.columns.astype(str)
+        df = df.droplevel(1)
         return df
 
+    def _generate_chisquared_test_df(self) -> pd.DataFrame:
+        """Return a dataframe for p-values values at each position for each cell line.
+
+        Returns:
+            pd.DataFrame: Chi-squared p-values for each position in each cell line
+        """
+        return self.individual_data.applymap(self._chisquared_test)
+
     @staticmethod
     def _find_diffs(means_df: pd.DataFrame, modes_df: pd.DataFrame) -> pd.DataFrame:
         """Find the differences between means and modes for each cell line at each pos
@@ -617,8 +636,11 @@ class GenomicPositionData:
 
         Raises:
             ValueError: invalid plotting backend
+            ValueError: Unable to plot more than 20 cell lines at once.
         """
 
+        MAX_GRAPHS = 20  # Sanity limit
+
         data = self.individual_data
         if not backend:
             backend = self.config.viz_backend
@@ -626,6 +648,11 @@ class GenomicPositionData:
         if cell_lines:
             data = data[data.index.isin(cell_lines)]
 
+        data = data.dropna(how="all", axis=1)
+
+        if len(data) > MAX_GRAPHS:
+            raise ValueError("Unable to plot more than 20 cell lines at once.")
+
         if backend == "plotly":
             fig: go.Figure = viz._make_stacked_fig(data, backend)
             fig.show()
@@ -654,7 +681,8 @@ class GenomicPositionData:
             height (int): figure height, defaults to 2000
             cell_lines (Optional[List[str]]): set of cell lines to analyze,
             defaults to all cell lines.
-            data_type (str): type of data to plot. Can to 'means', 'modes', or 'diffs'
+            data_type (str): type of data to plot.
+                            Can be 'means', 'modes', 'diffs', or 'pvalue'.
             backend (Optional[str]): plotting backend to override default
 
         Raises:
@@ -673,6 +701,9 @@ class GenomicPositionData:
         elif data_type == "diffs":
             data = self.diffs
             title_type = "Difference"
+        elif data_type == "pvalue":
+            data = self.allelic_data
+            title_type = "p-value"
         else:
             raise ValueError("Invalid data type")
 
@@ -754,9 +785,9 @@ class GenomicPositionData:
         """
 
         if cell_lines:
-            data = self.individual_data[self.individual_data.index.isin(cell_lines)]
+            data = self.allelic_data[self.allelic_data.index.isin(cell_lines)]
         else:
-            data = self.individual_data
+            data = self.allelic_data
 
         np.seterr(divide="ignore", invalid="ignore")  # ignore divide-by-zero errors
         sig_dict: Dict[str, List[ArrayLike]] = {
@@ -764,23 +795,18 @@ class GenomicPositionData:
             "position": [],
             "ad_stat": [],
             "p_crit": [],
-            "diff": [],
             "raw": [],
         }
+
         for index, row in data.iterrows():
             for column in row.index:
                 value = row[column]
-                if np.all(pd.notnull(value)):
-                    with warnings.catch_warnings():
-                        warnings.simplefilter("ignore", RuntimeWarning)
-                        good, stat, crits = self._anderson_darling_test(value)
-                    if good:
-                        sig_dict["diff"].append(self.diffs.loc[index, column])
-                        sig_dict["cellLine"].append(index)
-                        sig_dict["position"].append(column)
-                        sig_dict["raw"].append(value)
-                        sig_dict["ad_stat"].append(stat)
-                        sig_dict["p_crit"].append(crits[4])
+                if value and not np.isnan(value):
+                    sig_dict["cellLine"].append(index)
+                    sig_dict["position"].append(column)
+                    sig_dict["raw"].append(value)
+                    sig_dict["ad_stat"].append(value)
+                    sig_dict["p_crit"].append(1)
         sig_df = pd.DataFrame(sig_dict)
         return sig_df
 
@@ -804,7 +830,31 @@ class GenomicPositionData:
             return AD_stats(is_sig, stat, crits)
         return AD_stats(False, np.nan, [np.nan])
 
-    def return_individual_positions(self, cell_line: str) -> pd.DataFrame:
+    @staticmethod
+    def _chisquared_test(
+        data_list: List[int], cutoff: float = 0.001
+    ) -> Optional[float]:
+        """Perform Chi-squared analysis of a set of methylation calls.
+
+        Args:
+            data_list (List[int]): list of methylated (1)
+                                    and unmethylated (0) values for a read.
+            cutoff (float): pvalue cuttoff, defaults to 0.001
+
+        Returns:
+            Optional[float]: Chi-squared p-value if below cutoff, or None
+        """
+
+        if not np.all(np.isnan(data_list)):
+            my_data: NDArray[np.int_] = np.array(
+                [data_list.count(1), data_list.count(0)]
+            )
+            pvalue: float = stats.chisquare(my_data, axis=None)[1]
+            if pvalue <= cutoff:
+                return pvalue
+        return None
+
+    def _return_individual_positions(self, cell_line: str) -> pd.DataFrame:
         """Return a dataframe of methylation at genomic position resolution.
 
         Args:
@@ -814,72 +864,56 @@ class GenomicPositionData:
             pd.DataFrame: dataframe of methylation at genomic position resolution.
         """
 
-        class PointData(NamedTuple):
-            """Named tuple to cointain individual Quma point data."""
-
-            cell: str
-            read_position: int
-            read: str
-            positions: List[int]
-            values: List[str]
-
         def _find_positions(
-            cell: str, pos: str, data_list: List[quma.Reference], gseq: str
-        ) -> List[PointData]:
+            pos: str, data_list: List[quma.Reference], gseq: str
+        ) -> List[Tuple[List[int], List[str]]]:
             """Find individual resolution positions and data in quma results.
 
             Args:
-                cell (str): cell line.
                 pos (str): position of read.
                 data_list (List[quma.Reference]): list of quma read data
                 gseq (str): reference genomic sequence of read position
 
             Returns:
-                List[PointData]: list of data points from quma results.
+                List[Tuple[List[int], List[str]]]: list of data from quma results.
             """
-            output: List[PointData] = []
+            output: List[Tuple[List[int], List[str]]] = []
             for each in data_list:
-                read: str = each.fasta.com
-                offset: List[int] = [
-                    m.start() for m in re.finditer(each.res.gAli, gseq)
-                ]
-                real_off_set: int
-                if offset:
-                    real_off_set = offset[0]
-                else:
-                    real_off_set = 0
-                init_pos_list: List[int] = [
-                    m.start() for m in re.finditer("CG", each.res.gAli)
-                ]
+                try:
+                    offset: int = [
+                        m.start()
+                        for m in re.finditer(each.res.gAli.split("-")[0], gseq)
+                    ][0]
+                    # FIXME: This only matches initial substring in cases of gaps
+                except IndexError:
+                    offset = 0
+
                 pos_list: List[int] = [
-                    (this + int(pos) + real_off_set) for this in init_pos_list
+                    (m.start() + int(pos) + offset)
+                    for m in re.finditer("CG", each.res.gAli)
                 ]
 
-                my_point: PointData = PointData(
-                    cell=cell,
-                    read_position=int(pos),
-                    read=read,
-                    positions=pos_list,
-                    values=list(each.res.val),
-                )
+                my_point = (pos_list, list(each.res.val))
                 output.append(my_point)
 
             return output
 
-        def _collate_positions(output: List[PointData]) -> Dict[int, List[str]]:
+        def _collate_positions(
+            output: List[Tuple[List[int], List[str]]]
+        ) -> Dict[int, List[str]]:
             """Collect together list of quma data points into position-indexed dict.
 
             Args:
-                output (List[PointData]): list of quma read data points
+                output (List[Tuple[List[int], List[str]]]): list of read data points
 
             Returns:
                 Dict[int, str]: position-indexed genomic resolution methylation dict.
             """
             result_dict: Dict[int, List[str]] = {}
             for each in output:
-                for idx, val in enumerate(each.positions):
+                for idx, val in enumerate(each[0]):
                     try:
-                        my_val: str = each.values[idx]
+                        my_val: str = each[1][idx]
                     except IndexError:  # pragma: no cover
                         pass
                     if my_val in ["1", "0"]:
@@ -907,12 +941,11 @@ class GenomicPositionData:
         query: QumaResult = self.quma_results[cell_line]
         positions: List[str] = query._positions
         data: List[quma.Quma] = query.quma_output
-        big_list: List[List[PointData]] = []
+        big_list: List[Tuple[List[int], List[str]]] = []
         for pos, dat in zip(positions, data):
-            big_list.append(_find_positions(cell_line, pos, dat.data, dat._gseq))
+            big_list.extend(_find_positions(pos, dat.data, dat._gseq))
 
-        final_list: List[PointData] = [item for sublist in big_list for item in sublist]
-        full_data: Dict[int, List[str]] = _collate_positions(final_list)
+        full_data: Dict[int, List[str]] = _collate_positions(big_list)
 
         final_df: pd.DataFrame = _make_pos_df(full_data)
         return final_df
